package com.yihu.ehr.query;

import org.junit.Test;

/**
 * @author Sand
 * @version 1.0
 * @created 2016.02.18 9:27
 */
public class URLQueryBuilderTest {

    @Test
    public void testToString() throws Exception {
        /*URLQueryBuilder builder = new URLQueryBuilder();
        builder.addField("name")
                .addField("age")
                .addField("address")
                .addFilter("name", "?", "王", null)
                .addFilter("age", ">=", "20", null)
                .addSorter("name", true)
                .addSorter("age", false)
                .setPageNumber(20)
                .setPageSize(10);

        String url = builder.toString();
        Assert.assertTrue("编码失败", StringUtils.isNotEmpty(url));*/
    }
}